/*
 * @copyright		© Copyright 2006-2013, epages GmbH, All Rights Reserved.
 *
 * @module			de_epages/remotesearch/inc/resultpagehandler
 *
 * @revision		$Revision: 1.11.2.2.8.1 $
 */

define("de_epages/remotesearch/inc/resultpagehandler", [
	"jquery",
	"ep",

	"jquery/i18n",
	"jquery/fn/class",
	"ep/ajax",
	"ep/fn/busy",
	"ep/ui/form"
], function($, ep){

	return function(options){

		var
			// Init options
			o = $.extend({
				ajaxViewAction: "View",
				ajaxTemplateName: "Content",
				ajaxLanguageId: ep.config.languageId,
				ajaxPreventOnCategory: false,

				searchResultsId: "RemoteSearchResults",
				searchFacetsId: "RemoteSearchFacets",

				currency: $.i18n.settings.currency
			}, options),

			// Page content area
			page = $("body"),

			// Navbars (left / right)
			navbar = $(".NavBarLeft > .PropertyContainer > .SizeContainer, .NavBarRight > .PropertyContainer > .SizeContainer"),

			// Category tree box
			category = navbar.children(".CategoriesBox"),

			// Search results area
			results = $("#" + o.searchResultsId),

			// Search facets area
			facets = $("#" + o.searchFacetsId),

			// Form of facets
			facetsForm = facets.find("form"),

			// Timeout id holder
			delayTimout,

			// Reset trigger delay for auto submit
			clearSubmit = function () {
				clearTimeout(delayTimout);
			},

			// Delayed auto sumbit
			delaySubmit = function(){
				clearSubmit();

				// Delay submit after user input handling
				delayTimout = setTimeout(function(){
					facetsForm.trigger("submit");
				}, 1800);
			};

		// Set factets area position
		if( navbar.length ){
			if (category.length){
				if (jQuery.contains(navbar.first()[0], category[0])) {
					navbar = navbar.first();
				}
				else if (jQuery.contains(navbar.last()[0], category[0])) {
					navbar = navbar.last();
				}

				category.remove();
			}
			else {
			navbar = navbar.first();
			}

			navbar
				.prepend(facets);

			navbar
				.children(".SearchBox, .SearchElement")
				.first()
				.prependTo(navbar);

			navbar
				.children()
				.removeClass(/NavBarElement\d+/)
				.each(function(i, node){
					$(this).addClass("NavBarElement"+i);
				});
		}
		else {
			//facets.addClass("NavBarMiddle");
            facets.prependTo(".ContentAreaWrapper");
		}

		// Init AJAX refresh
		if (window.history && history.pushState && history.replaceState && 'state' in history) {
			var
				firstPopstate = true,

				locationUrl = "",

				ajaxUrl = "",

				// Prepare facets form for AJAX load
				facetAjaxInit = function(){
					var self = facetAjaxInit;

					if (!self.cache) {
						self.cache = $('<div/>');
						self.jsonSnippet = $('<div><input type="hidden" name="TemplateAction" value="' + o.ajaxViewAction + '"/><input type="hidden" name="TemplateName" value="' + o.ajaxTemplateName + '"/><input type="hidden" name="TemplateLanguageID" value="' + o.ajaxLanguageId + '"/></div>');
					}
					self.viewAction = facetsForm.find("input[name=ViewAction]");

					// Clear
					self.viewAction.val(o.ajaxViewAction);
					self.jsonSnippet.prependTo(self.cache);

					// Set pushstate url
					locationUrl = "?"+facetsForm.serialize();

					// Set ajax properties
					self.viewAction.val("JSONSnippet");
					self.jsonSnippet.prependTo(facetsForm);

					ajaxUrl = "?"+facetsForm.serialize();
				},

				facetAjaxReset = function () {
					var self = facetAjaxInit;

					if (self.cache) {
						// Clear
						self.viewAction.val(o.ajaxViewAction);
						self.jsonSnippet.prependTo(self.cache);
					}

					facetsForm.uiForm("option", "ajax", false);
				},

				win = $(window);

			// Init current ajax and location url
			facetAjaxInit();

			// Set current history data
			history.replaceState({
				snippetUrl: ajaxUrl
			}, null, location.href);

			// Register history event
			win.on("popstate", function(event) {
					if (firstPopstate) {
						// Reset prent ajax on first popstate
						firstPopstate = false;

						return;
					}

					var state = history.state;
					// Hide busy layer on page
					page.busy("show");

					// Load state by ajax
					ep.ajax({
						url: state.snippetUrl,
						dataType: "json"
					})
					.done(function( data, status, jqXHR ){
						var snippet = $("<div/>").append(data.snippet);

						// Replace page content
						results
							.empty()
							.html(snippet.find("#" + o.searchResultsId).html() || snippet.html() || "")
							.find(".ep-js")
							.metaparse();

						facetsForm
							.empty()
							.html(snippet.find("#" + o.searchFacetsId + " form").html() || "")
							.find(".ep-js")
							.metaparse();

						// Derenference
						snippet = undefined;

						// Hide busy layer on page
						page.busy("hide");
					})
					.fail(function () {
						// Hide busy layer on page
						page.busy("hide");
					});
				});

			// Init form for auto AJAX
			facetsForm
				.uiForm({
					ajax: function(){
						// Show busy layer on page
						page.busy("show");

						// Manipulate form for template loading via AJAX
						facetAjaxInit();

						// Push history state
						history.pushState({
							snippetUrl: ajaxUrl
						}, null, locationUrl);

						// Reset prent ajax on first popstate
						firstPopstate = false;

						// Trigger popstate for last push
						win.trigger("popstate");
					}
				});
		}
		else {
			// set hidden input for ViewAction to default Faceted-ViewAction if only "View" was found
			self.viewAction = facetsForm.find("input[name=ViewAction]");
			if(self.viewAction.val()==="View"){
				self.viewAction.val("ViewFaceted");
			}

			facetsForm.uiForm();
		}

		// Register input event handling
		facetsForm
			.on("change", "input:not(:text)", function(){
				var elem = $(this);

				if (o.ajaxPreventOnCategory && elem.is(".RemoteSearchFacetCategory *")) {
					// Reset AJAX setup
					if (typeof facetAjaxReset !== "undefined") {
						facetAjaxReset();
					}

					// Change ObjectID for new category context
					facetsForm.find("input[name=ObjectID]").val(elem.val());
				}

				facetsForm.trigger("submit");
			})
			.on("keyup", "input[name] + input:text", function(){
				var input = $(this),
					slide = input.closest("li").find(".ui-slider"),
					val = input.val(),
					value = $.i18n.parseNumber(val),
					values = slide.slider("option", "values");
				values[input.is(".FacetRangeMin") ? 0 : 1] = value;

				// Set value to named (hidden) input
				input.prev().val( val );

				// Set value to slider
				if( value !== NaN ){
					slide.slider("option", "values", values);
				}
			})
			.on("slidestop", ".ui-slider", delaySubmit)
			.on("slide", ".ui-slider", function(){
				var slide = $(this),
					values = slide.slider("values"),
					format = slide.data("format") || 'n',
					vals = [$.i18n.formatNumber(values[0], format), $.i18n.formatNumber(values[1], format)],
					input = slide.closest("li").find("input");

				input.filter(".FacetRangeMin").val( vals[0] );
				input.filter(".FacetRangeMax").val( vals[1] );

				input.trigger('focus')		// Trigger non native placeholder handling
					.trigger('validate'); // Validate values

				clearSubmit();
			})
			.on("click", "a.RemoteSearchFacetsReset", function(){
				var block = $(this).closest('.RemoteSearchFacet, form');

				block.find("input:checkbox, input:radio")
					.attr("checked", false);

				block.find("input:text")
					.val("")
					.trigger("keyup");

				facetsForm.trigger("submit");
			});
	};

});