/*
 * @copyright		© Copyright 2006-2012, epages GmbH, All Rights Reserved.
 *
 * @module			ep.ui.core
 *
 * @revision		$Revision: 1.8.12.1 $
 */

define("ep/ui/core", [
	"jquery",
	"ep",
	"util/scope"
], function ($, ep, scope) {

	var ui = scope("ep.ui"),

		fx,

		getId = function( selector ){
			var id = "";

			if( typeof selector == "string" ){
				id = selector;
			}
			else if( selector instanceof $ ){
				id = selector.selector;
			}

			return id || "Expando:" + (selector[ $.expando ] || (selector[ $.expando ] = ++$.guid));
		},

		getElemData = function( item, stack ){
			var type = $.type(item),
				itemData = {},
				data;

			if( type == "string" ){
				data = {
					src: item,
					desc: ""
				};
			}
			else if( item.nodeType ){
				var elem = $(item);

				if( elem.is("img") ){
					itemData = elem.data();

					// get href from parent node if available
					itemData.href = elem.closest("a").attr("href");

					data = {
						node: item,
						src	: elem.attr("src"),
						desc: (elem.attr("title") || elem.attr("alt")).replace(/\\\\n/g,"\n")
					};
				}
			}
			else if( type == "object" && item.src ){
				itemData = item;
				data = {
					src: item.src,
					desc: item.desc || ""
				};
			}

			if( data ){
				data.srcL	= itemData.srcL || itemData.srcM || data.src;
				data.srcMl	= itemData.srcMl || data.srcL;
				data.srcM	= itemData.srcM || data.srcMl;
				data.srcMs	= itemData.srcMs || data.srcM;
				data.srcS	= itemData.srcS || data.srcM;
				data.srcXs	= itemData.srcXs || data.srcS;

				data.href	= itemData.href || data.srcL;

				stack.push(data);
			}
		};

	ui.imgData = function( selector ){
		var id	= getId(selector),
			result = ui.imgData.cache[ id ];

		if( !result ){
			var type = $.type(selector),
				data = [];

			if( type == "string" && !(/\//).test(selector) ){
				$(selector).each(function(){
					getElemData( this, data );
				});
			}
			else if( type == "array" ){
				$.each( selector, function( i, item ){
					getElemData( item, data );
				});
			}
			else if( selector instanceof $ ){
				selector.each(function(){
					getElemData( this, data );
				});
			}
			else if( type == "string" || (type == "object" && (selector.src || selector.nodeType)) ){
				getElemData( selector, data );
			}

			result = new ui.imgData.init(data);

			result.on("location", function(){
				var href = (result.current() || {}).href;
				if( href ){
					location.href = href;
				}
			});

			if( id ){
				ui.imgData.cache[ id ] = result;
			}
		}

		return result;
	}

	ui.imgData.cache = {};

	ui.imgData.init = function( data ){
		var self = this;

		self.eventObject = $('<div/>');

		$(window)
			.on("resize",function(){
				// trigger resize
				self.trigger("resize");

				// clear trigger delay
				clearTimeout(self._resizestopdelay);

				// trigger a special event when resize stopped
				self._resizestopdelay = setTimeout(function(){
					self.trigger("resizestop");
				},100);
			});

		return $.merge( self, data );
	};

	ui.imgData.init.prototype = {

		length: 0,

		currentItem: 0,

		each: $.fn.each,

		get: function( i ){
			return arguments.length ? this[ i ] : $.merge([],this);
		},

		current: function( i ){
			if( arguments.length ){
				var set = this.currentItem;

				if( i == 'next' ){
					set++;
				}
				else if( i == 'prev' ){
					set--;
				}
				else{
					set = i;
				}

				if( set == this.currentItem || set >= this.length || set < 0 ){
					return this;
				}
				else{
					this.currentItem = set;
					return this.trigger('change');
				}
			}
			else{
				return this.get(this.currentItem);
			}
		},

		prev: function(){
			return this.get(this.currentItem-1);
		},

		next: function(){
			return this.get(this.currentItem+1);
		},

		first: function(){
			return this.get(0);
		},

		last: function(){
			return this.get(this.length-1);
		},

		on: function( events, handler ){
			this.eventObject.on( events, $.proxy(handler,this) );
			return this;
		},

		off: function( events, handler ){
			if( handler ){
				this.eventObject.off( events, $.proxy(handler,this) );
			}
			else{
				this.eventObject.off( events );
			}
			return this;
		},

		trigger: function( events ){
			this.eventObject.trigger( events );
			return this;
		}
	};

	// Create a
	ui.createMoveToTop = function (elementName) {
		return function( event, silent ) {
			this[elementName].nextAll(":visible").insertBefore( this[elementName] )

			return this;
		}
	};

	// center an image
	ui.imgCenter = function( img, fitToContainer ){
		var container	= img.parent(),
			node		= img[0] || {}
			maxWidth	= container.innerWidth(),
			maxHeight	= container.innerHeight(),
			imgWidth	= node.width || node.width || 1,	// pre access to make width/height available => webkit bug
			imgHeight	= node.height || 1,
			imgCSS		= {
				position: "absolute"
			},
			newWidth	= imgWidth,
			newHeight	= imgHeight;

		if( imgWidth / maxWidth >= imgHeight / maxHeight ){
			if( fitToContainer ){
				newWidth		= Math.min(imgWidth,maxWidth);
				imgCSS.width	= newWidth+"px";
				imgCSS.height	= "auto";
			}
			imgCSS.top		= ((maxHeight - (newWidth * imgHeight / imgWidth )) / 2) + "px";
			imgCSS.left		= ((maxWidth - newWidth) / 2) + "px";
		}
		else{
			if( fitToContainer ){
				newHeight		= Math.min(imgHeight,maxHeight);
				imgCSS.width	= "auto";
				imgCSS.height	= newHeight+"px";
			}
			imgCSS.top		= ((maxHeight - newHeight) / 2) + "px";
			imgCSS.left		= ((maxWidth - (newHeight * imgWidth / imgHeight )) / 2) + "px";
		}

		img.css(imgCSS);
	};


	// change-over effects
	$.extend(
		(fx = ui.changeOverFx = function( effect ){
			/* TODO: random
			if( effect == "random" ){

			}
			*/

			return fx[ effect ] || fx["default"];
		}), {
			"fade": function( container, from, to, duration, callback ){
				to.insertBefore(from)
					.fadeIn(1000);

				from.fadeOut(1000, callback);
			}
		}, {
			"fade-centered": function( container, from, to, duration, callback ){
				ui.imgCenter(to.find("img"), true);

				to.insertBefore(from)
					.fadeIn(1000);

				from.fadeOut(1000, callback);
			}
		}
	);

	// Set fade as default
	fx["default"] = fx.fade;

	return ep;

});