/*
 * @copyright		© Copyright 2006-2010, epages GmbH, All Rights Reserved.
 *
 * @module			ep.ui.spinner
 *
 * @revision		$Revision: 1.11.2.2.2.1 $
 */

define("ep/ui/spinner", [
	"jquery",
	"ep",

	"ep/i18n",
	"ep/date",
	"ep/sprite",
	"ep/ui/validate"
], function ($, ep) {

/*
 * @dictionary			ep.dict
 *
 * @translation			{INVALID_STEP}
 */

	var is = function( val ){
			return !!( val===0 || (!isNaN(val) && val) );
		},
		html = $('html');

	$.widget('ui.uiSpinner',$.ui.uiValidate,{

		options: {
	// ep.ui.Input
		//	big:				true,
		//	autofocus:			false,
		//	info:				'text',

	// ep.ui.Validate
		//	accept:				'',
		//	min:				1,
		//	max:				20,
		//	minlength:			1,
		//	maxlength:			4,
		//	pattern:			[\w]+,
		//	required:			true,
		//	tooltipOffset:		[0,-7],
		//	valid:				true,
		//	type:				'basic',
		//	format:				'',

	// ep.ui.Spinner
		//	unit:				'GByte',
			type:				'number',
			format:				'n0',
			step:				1,
			stepType:			'hours',
			validateStep: false
		},

		_create: function(){
			this._superApply(arguments);
			var self = this;

			this.options = $.extend({
					step:			parseInt( this.elem.attr('step'), 10 )
				},this.options);

			this.elem
				.addClass('ep-uiSpinner')
				.on( 'keydown', function(event){
					if( event.keyCode === 38 || event.keyCode === 40 ){
						event.preventDefault();
						if( !self._interval ){
							self._start( event.keyCode === 38 ? 1 : -1 );
						}
					}
				})
				.on( 'keyup', function(event){
					if( event.keyCode === 38 || event.keyCode === 40 ){
						event.preventDefault();
						self._stop();
					}
				});

			this.stepper = $('<span>')
				.addClass('ep-uiInput ep-uiInput-button ep-uiSpinner-stepper')
				.insertAfter(this.elem);
			this.stepUp = ep( this.dict.parse('<a href="javascript:;">+</a>') )
				.addClass('ep-uiSpinner-stepUp')
				.attr({
					'tabIndex': -1
				})
				.on( 'mousedown.uiSpinner', function(event){
					if(event.button === 0){
						self._start(1);
					}
				})
				.appendTo(this.stepper);
			this.stepDown = ep( this.dict.parse('<a href="javascript:;">&ndash;</a>') )
				.addClass('ep-uiSpinner-stepDown')
				.attr({
					'tabIndex': -1
				})
				.on( 'mousedown.uiSpinner', function(event){
					if(event.button === 0){
						self._start(-1);
					}
				})
				.appendTo(this.stepper);

			this.stack.push( this.stepper[0], this.stepUp[0], this.stepDown[0] );
		},

		_init: function(){
			this._superApply(arguments);
			this._unitInit();

			var o = this.options;

			// Detect format by format,step, min and max options
			if( o.type!=='date' ){
				var formatMatch = (/(c|n|p)(\d{0,})/).exec(o.format),
					formatDecimals = parseInt(formatMatch[2] || 0, 10),

					stepSplit = (o.step+'').split('.'),
					stepDecimals = stepSplit[1] ? stepSplit[1].length : 0,

					minSplit = (o.min+'').split('.'),
					minDecimals = minSplit[1] ? minSplit[1].length : 0,

					maxSplit = (o.max+'').split('.'),
					maxDecimals = maxSplit[1] ? maxSplit[1].length : 0,

					// Get highest value of decimals from format/step/min/max
					decimals = Math.max(Math.max(Math.max(formatDecimals, stepDecimals), minDecimals), maxDecimals);

				this.options.format = formatMatch[1] + decimals;
			}
		},

		_unitInit: function(){
			if( !this.unit && this.options.unit ){
				this.unit = $('<span>')
					.addClass('ep-uiSpinner-unit')
					.insertBefore(this.stepper);
				this.stack.push(this.unit[0]);
			}
			if( this.unit ){
				this.unit.text(this.options.unit);
				var padding = (this.unit.width()+4)+'px';
				this.elem.css( 'padding-right', padding );
				this.unit.css( 'margin-left', '-' + padding );
			}
		},

		_tooltipInit: function(){
			this._superApply(arguments);
			if( this.tooltip ){
				this.tooltip.uiTooltip( 'option', 'offsetAdjust', [30,-1] );
			}
		},

		_changeAttr: function( event, attr ){
			var self = this;
			$.each( attr, function( name, value ){
				if( (/^(accept|pattern|required)$/i).test(name) ){
					self.options[ name ] = value;
				}
				else if( (/^(min|max|minlength|maxlength|step)$/i).test(name) ){
					self.options[ name ] = parseInt( value, 10 );
				}
			});
		},

		_parseValue: function(){
			var val = this.elem.attr('value'),
				o = this.options,
				region = {region: o.region};

			if( o.type === 'date' ){
				if( !is(this._val) ){
					this._val = is(o.min) ? new ep.Date(o.min) : new ep.Date();
				}
				val = $.i18n.parseDate( val, o.format, region );
				this._val = is(val) ? new ep.Date(val) : this._val;
			}
			else{
				if( !is(this._val) ){
					this._val = is(o.min) ? o.min : 0;
				}
				region.currency = o.currency;

				val = $.i18n.parseNumber( val, 10, region );
				this._val = is(val) ? val : this._val;
			}
		},

		_start: function( inc ){
			this._parseValue();

			this._type = '_spin' + (this.options.type==='date' ? 'Date' : 'Number');
			this._step = this.options.step * inc;
			this._count = 0;
			this[this._type]();
			// set interval for auto count up/down
			this._interval = setInterval( $.proxy(this, '_spin'), 250 );

			html.on( 'mouseup.uiSpinner', $.proxy(this, '_stop') );
		},

		_spin: function(){
			this._count++;
			if( (/^(10|20|30)$/).test(this._count+'') ){
				clearInterval(this._interval);
				// update interval speed for auto count up/down
				this._interval = setInterval( $.proxy(this, '_spin'), 150/this._count );
			}
			this[this._type]();
		},

		_spinNumber: function(){
			var o = this.options;

			this._val += this._step;

			if( is(o.min) && o.min > this._val ){
				this._val = o.min;
				this._stop();
			}
			else if( is(o.max) && o.max < this._val ){
				this._val = o.max;
				this._stop();
			}

			this.elem.val( $.i18n.formatNumber( this._val , o.format, {region:o.region,currency:o.currency} ) );
		},

		_spinDate: function( add ){
			var o = this.options;

			this._val[ $.camelCase('add-'+o.stepType) ]( this._step );

			if( is(o.min) && o.min > this._val.getTime() ){
				this._val = new ep.Date(o.min);
				this._stop();
			}
			else if( is(o.max) && o.max < this._val.getTime() ){
				this._val = new ep.Date(o.max);
				this._stop();
			}

			this.elem.val( $.i18n.formatDate( this._val, o.format, {region:o.region} ) );
		},

		_stop: function(){
			html.off( 'mouseup.uiSpinner', $.proxy(this, '_stop') );

			this._interval = clearInterval(this._interval);


			this.elem
				.filter(':visible')
					.focus()
				.end()
				.trigger('changeValue');
		},

		_createErrorMsgOptions: function(){
			var exampleValue = '',
				self = this,
				o = self.options;
			var tmpOptions = this._superApply(arguments);
			if(this.valid == "INVALID_STEP"){
				exampleValue = o.min + ', ' + (o.min + o.step) + ', ' + (o.min + 2 * o.step);

				return {
					example: exampleValue
				};
			}
			else{
				return tmpOptions;
			}
		},

		_validate: function( event ){
			var self = this,
				o = self.options,
				val = $.i18n.parseNumber(this.elem.val());
			this._superApply(arguments);
			if(o.validateStep && this.valid == true){
				if( ((val - o.min).toFixed(3) * 1000 ) % (o.step * 1000) ){
					this._setValidStatus("INVALID_STEP");
					if( event.type!=='changeAttr' ){
						$(this.elem[0].form).trigger('change');
					}
				}
			}
		},

		destroy: function(){
			this.elem
				.off('.uiSpinner')
				.removeClass('epWidth-29')
				.removeClass(/ep-uiSpinner[\w\d-]*/g);

			this._superApply(arguments);
		}
	});

	return ep;

});